@@ -1,29 +1,28 @@
-makeDsim <- function(pedigree, N, invertD = FALSE, calcSE = FALSE)
+makeDsim <- function(pedigree, N, parallel = FALSE, ncores = getOption("cores"), invertD = FALSE, calcSE = FALSE, returnA = FALSE)
 {
   numeric.pedigree <- numPed(pedigree)
   n <- dim(pedigree)[1]
-  alleles <- matrix(-998, nrow = n, ncol=2) 
+  alleles <- matrix(as.integer(-998), nrow = n, ncol=2) 
   dfounders <- which(numeric.pedigree[, 2] == -998)
   sfounders <- which(numeric.pedigree[, 3] == -998)
   uniqp <- c(unique(numeric.pedigree[, 2])[-1], unique(numeric.pedigree[, 3])[-1])
-unique(numeric.pedigree[, 3])[-1]
   ndfounders <- length(dfounders)
   nsfounders <- length(sfounders)
 
   alleles[dfounders, 1] <- as.integer(seq(1, ndfounders, 1)) 
   alleles[sfounders, 2] <- as.integer(seq(ndfounders+1, (ndfounders + nsfounders), 1))
   nonfounders <-  which(alleles == -998)
-  segregation <- matrix(ceiling(runif(length(nonfounders)*N, min = 0, max = 2)),ncol=N)
+  segregation <- matrix(as.integer(ceiling(runif(length(nonfounders)*N, min = 0, max = 2))),ncol=N)
   dalleles <- matrix(alleles[,1], nrow = dim(alleles)[1], ncol = N)
   salleles <- matrix(alleles[,2], nrow = dim(alleles)[1], ncol = N)
 
-  approxD.tmp <- makeD(pedigree, invertD = invertD)
+  approxD.tmp <- makeD(pedigree, parallel = parallel, ncores = ncores, invertD = invertD, returnA = returnA)
   approxD <- sm2list(approxD.tmp$D, colnames = c("row", "column", "D"))
   D.nonself <- which(approxD[,3] < 1) 
   approxD.nonself <- approxD[D.nonself,]
   simD.nonself <- matrix(NA, nrow = dim(approxD.nonself)[1], ncol = N) 
 
-print("starting to make Dsim")
+cat(paste("making Dsim ..."))
     start <- 1
     order.ind <- order(uniqp)
     uniqp <- uniqp[order.ind]
@@ -36,10 +35,10 @@ print("starting to make Dsim")
       lmom <- length(mothered)
       lfath <- length(fathered)
       segregation.tmp <- matrix(segregation[start:(start - 1 + lmom + lfath), ], ncol = N)
-      jgametes <- matrix(vapply(1:(lmom+lfath), FUN = vchoose, FUN.VALUE = vector("numeric", N), N = N, which2 = segregation.tmp, values2 = jalleles), ncol = N, byrow = TRUE) 
+      jgametes <- matrix(as.integer(vapply(1:(lmom+lfath), FUN = vchoose, FUN.VALUE = vector("numeric", N), N = N, which2 = segregation.tmp, values2 = jalleles)), ncol = N, byrow = TRUE) 
       start <- start + lmom + lfath 
-     if(!is.null(mothered)) dalleles[mothered, ] <- jgametes[1:lmom, ]
-     if(!is.null(fathered)) salleles[fathered, ] <- jgametes[(lmom + 1):lfath, ]
+     if(!is.null(mothered)) dalleles[mothered, ] <- as.integer(jgametes[1:lmom, ])
+     if(!is.null(fathered)) salleles[fathered, ] <- as.integer(jgametes[(lmom + 1):lfath, ])
     }
 
   stack.alleles<-rbind(dalleles, salleles)
@@ -60,15 +59,13 @@ print("starting to make Dsim")
   Dsim@i<-as.integer(Dsim.row[order.index]-1)
   Dsim@p<-as.integer(c(match(1:n, Dsim.col[order.index]), length(order.index)+1)-1)
   Dsim@x<-Dsim.x[order.index]
-print("Dsim made")
+cat(paste(".done", "\n"))
   logDetDsim <- determinant(Dsim, logarithm = TRUE)$modulus[1]
 
  
   if(invertD){
-    print("starting to invert Dsim")
     Dsiminv<-solve(Dsim)
     Dsiminv@Dimnames <- list(pedigree[,1], NULL)
-    print("done inverting Dsim")
     listDsiminv<-sm2list(Dsiminv, rownames=pedigree[,1], colnames=c("row", "column", "simDinverse"))
     Dsim <- as(Dsim, "dgCMatrix")
  return(list(A=approxD.tmp$A, D=approxD.tmp$D, logDetD = approxD.tmp$logDet, Dinv=approxD.tmp$Dinv, listDinv=approxD.tmp$listDinv, Dsim=Dsim, logDetDsim = logDetDsim, Dsiminv=Dsiminv, listDsim=listDsim, listDsiminv=listDsiminv))
